#!/usr/bin/env bash
#		GMT EXAMPLE 31
#
# Purpose:	Illustrate usage of non-default fonts in PostScript
# GMT modules:	set, coast, plot, text, legend
# Unix progs:	awk, cat, rm
#

# Add custom code from file gmtadd.ps to GMT ps file
embedpscode()
{
 local in="$1"
 local tmp=`mktemp`

 gawk 'BEGIN {p=1}
 /\/PSL_font_encode / {p=0}
 {if(p==1) print}
 ' "$in" >"$tmp"
 if [ -f gmtadd.ps ]; then
  cat gmtadd.ps >>"$tmp"
 fi
 gawk 'BEGIN {p=0}
 /\/PSL_font_encode / {p=1}
 {if(p==1) print}
 ' "$in" >>"$tmp"
 mv "$tmp" "$in"
}

# set AWK to awk if undefined
AWK=${AWK:-awk}

# Create gmtadd.ps and PSL_custom_fonts.txt for font BZar
g++ -o queryfont `pkg-config --cflags --libs freetype2` queryfont.cpp

./mktable BZar.ttf farsi.txt BZFARSI >gmtadd.ps
./mktable --notable BZarBd.ttf farsi.txt BZFARSI >>gmtadd.ps # Encoding vector is same as for BZar, so --notable

# Several fonts can be used
./mktable DejaVuSans farsi.txt DVFARSI >>gmtadd.ps # Find font DejaVuSans in system and create a different encoding vector for it
./mktable --notable DejaVuSans-Bold farsi.txt DVFARSI >>gmtadd.ps

./gensed farsi.txt >uni2oct # This is font independent sed converter

gmt begin ex31
	# Set FONTPATH used in image conversion
	# $(dirname 0) is the path to the current bash script
	gmt set PS_CONVERT="C-sFONTPATH=."

	# common settings
	gmt set FORMAT_GEO_MAP ddd:mm:ssF MAP_DEGREE_SYMBOL colon MAP_TITLE_OFFSET 20p \
		MAP_GRID_CROSS_SIZE_PRIMARY 0.4c PS_LINE_JOIN round\
		FONT 10p,BZarBZFARSI FONT_TITLE 24p,BZarBZFARSI MAP_ANNOT_OBLIQUE lon_horizontal,lat_parallel,tick_extend

	# map of countries
	gmt coast -R44/64/23/41 -JM10 -B5 -A250 \
		-Slightblue -Glightgreen -W0.25p -N1/1p,white
	gawk 'BEGIN {FS=","}
	$4 > 1 { cmd="./ar.py \""$3"\"|sed -r -f uni2oct"
	 cmd | getline out
	 close(cmd)
	 print $1,$2,out
	}
	' data.csv | gmt text -F+f7p,BZarBoldBZFARSI -Dj0.1c -Gwhite
	echo "53 34 $(./ar.py "این متن با قلم DejaVu ساخته شده است."|sed -r -f uni2oct)" | gmt text -F+f8p,DejaVuSans-BoldDVFARSI -Dj0.1c -Gwhite

	# Patching EPS file to provide correct encoding vector for the font BZar
	n=`ls -1 ~/.gmt/sessions|wc -l` # Number of gmt sessions
	if [ $n -ne 1 ]; then
	 echo Can\'t find GMT session # Don't know how to get right dir
	else
	 embedpscode ~/.gmt/sessions/*/gmt_0.ps-
	fi
gmt end

rm queryfont gmtadd.ps uni2oct PSL_custom_fonts.txt
