#include <ft2build.h>
#include <iostream>
#include <set>
#include FT_FREETYPE_H

// Count elements in array
size_t count(FT_UInt32 *array) {
  if (array == nullptr)
    return 0;
  size_t ret = 0;
  while (array[ret] != 0)
    ret++;
  return ret;
}

// One argument - font file
// Output - pairs (unicode name)
int main(int argc, char **argv) {
  if (argc != 2)
    return 1;

  FT_Library library;
  FT_Face face;
  const size_t bufsize = 1000;
  char buf[bufsize];
  std::set<FT_UInt> coded; // To search non-unicode glyphs

  auto error = FT_Init_FreeType(&library);
  if (error) {
    std::cout << "Can't init Freetype library" << std::endl;
    return 1;
  }

  error = FT_New_Face(library, argv[1], 0, &face);
  if (error) {
    std::cout << "Can't open font file: " << argv[1] << std::endl;
    return 1;
  }

  error = FT_Select_Charmap(face, FT_ENCODING_UNICODE);
  if (error) {
    std::cout << "Can't select unicode charmap" << std::endl;
    return 1;
  }

  FT_ULong charcode;
  FT_UInt gindex;

  charcode = FT_Get_First_Char(face, &gindex);
  while (gindex != 0) {
    FT_Get_Glyph_Name(face, gindex, buf, bufsize);
    auto variants = count(FT_Face_GetVariantsOfChar(face, charcode));
    if (variants == 0)
      std::cout << std::hex << charcode << std::dec << " " << buf << std::endl;
    else
      std::cout << std::hex << charcode << std::dec << " " << buf << " "
                << variants << std::endl;
    coded.insert(gindex);
    charcode = FT_Get_Next_Char(face, charcode, &gindex);
  }

  /* Search non-unicode glyphs (variants, ligatures, custom symbols, etc.)
  for (FT_Long i = 0; i < face->num_glyphs; i++) {
    error = FT_Get_Glyph_Name(face, i, buf, bufsize);
    if (error) {
      std::cout << i << " invalid" << std::endl;
      continue;
    }
    if (coded.count(i) > 0)
      std::cout << i << " unicode with name " << buf << std::endl;
    else
      std::cout << i << " private with name " << buf << std::endl;
  }
  */

  FT_Done_Face(face);
  FT_Done_FreeType(library);
  return 0;
}
